# -------------------------------
# SPDX-License-Identifier: ISC
#
# Copyright © 2021 Daniel Wolf <<nephatrine@gmail.com>>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
# -------------------------------

include(NXIncludeGuard)
include(NXScopePropagate)

nx_include_guard()

# ===================================================================

set(NXGenerateExportHeader_DIRECTORY "${CMAKE_CURRENT_LIST_DIR}")

function(nx_generate_export_header sBaseName)
	nx_scope_initialize()

	string(TOUPPER "${sBaseName}" sBaseName)
	string(MAKE_C_IDENTIFIER "${sBaseName}" sBaseName)
	list(APPEND _NX_PROPAGATE_LOCAL "${sBaseName}_SHARED_DEFINE" "${sBaseName}_STATIC_DEFINE")

	set(GEH_BASE_NAME ${sBaseName})
	set(GEH_DEFINE_NO_DEPRECATED 0)

	unset(GEH_CUSTOM_CONTENT)
	unset(GEH_CUSTOM_CONTENT_FROM_VARIABLE)
	unset(GEH_DEPRECATED_MACRO_NAME)
	unset(GEH_EXPORT_FILE_NAME)
	unset(GEH_EXPORT_MACRO_NAME)
	unset(GEH_EXPORT_MACRO_CNAME)
	unset(GEH_INCLUDE_GUARD_NAME)
	unset(GEH_NO_DEPRECATED_MACRO_NAME)
	unset(GEH_NO_EXPORT_MACRO_NAME)
	unset(GEH_PREFIX_NAME)
	unset(GEH_STATIC_DEFINE)
	unset(GEH_DEFINE_SYMBOL)

	set(lsParseModes
		BASE_NAME
		EXPORT_MACRO_NAME
		EXPORT_MACRO_CNAME
		EXPORT_FILE_NAME
		DEPRECATED_MACRO_NAME
		NO_EXPORT_MACRO_NAME
		INCLUDE_GUARD_NAME
		STATIC_DEFINE
		NO_DEPRECATED_MACRO_NAME
		PREFIX_NAME
		CUSTOM_CONTENT_FROM_VARIABLE
		DEFINE_SYMBOL)
	set(lsParseToggles DEFINE_NO_DEPRECATED)
	set(sParseMode "EXPORT_FILE_NAME")

	foreach(sArgument ${ARGN})
		if("${sArgument}" IN_LIST lsParseModes)
			set(sParseMode "${sArgument}")
		elseif("${sArgument}" IN_LIST lsParseToggles)
			set(GEH_${sArgument} 1)
		else()
			set(GEH_${sParseMode} "${sArgument}")
		endif()
	endforeach()

	if(NOT DEFINED GEH_EXPORT_FILE_NAME)
		string(TOLOWER "${GEH_BASE_NAME}_export.h" GEH_EXPORT_FILE_NAME)
	endif()
	if(NOT DEFINED GEH_INCLUDE_GUARD_NAME)
		get_filename_component(GEH_INCLUDE_GUARD_NAME "${GEH_EXPORT_FILE_NAME}" NAME)
		string(TOUPPER "${CMAKE_PROJECT_NAME}" GEH_INCLUDE_GUARD_PREFIX)
		string(TOUPPER "${GEH_INCLUDE_GUARD_NAME}" GEH_INCLUDE_GUARD_NAME)
		if(NOT GEH_INCLUDE_GUARD_NAME MATCHES "^${GEH_INCLUDE_GUARD_PREFIX}")
			set(GEH_INCLUDE_GUARD_NAME "${GEH_INCLUDE_GUARD_PREFIX}_${GEH_INCLUDE_GUARD_NAME}")
		endif()
		string(MAKE_C_IDENTIFIER "${GEH_INCLUDE_GUARD_NAME}" GEH_INCLUDE_GUARD_NAME)
		set(GEH_INCLUDE_GUARD_NAME "${GEH_PREFIX_NAME}${GEH_INCLUDE_GUARD_NAME}")
	endif()

	if(NOT DEFINED GEH_STATIC_DEFINE)
		if(DEFINED ${sBaseName}_STATIC_DEFINE)
			set(GEH_STATIC_DEFINE "${${sBaseName}_STATIC_DEFINE}")
		else()
			set(GEH_STATIC_DEFINE "${GEH_PREFIX_NAME}${GEH_BASE_NAME}_STATIC_DEFINE")
		endif()
	endif()
	if(NOT DEFINED ${sBaseName}_STATIC_DEFINE)
		set(${sBaseName}_STATIC_DEFINE "${GEH_STATIC_DEFINE}")
	endif()

	if(NOT DEFINED GEH_DEFINE_SYMBOL)
		if(DEFINED ${sBaseName}_SHARED_DEFINE)
			set(GEH_DEFINE_SYMBOL "${${sBaseName}_SHARED_DEFINE}")
		else()
			set(GEH_DEFINE_SYMBOL "${GEH_PREFIX_NAME}${GEH_BASE_NAME}_EXPORTS")
		endif()
	endif()
	if(NOT DEFINED ${sBaseName}_SHARED_DEFINE)
		set(${sBaseName}_SHARED_DEFINE "${GEH_DEFINE_SYMBOL}")
	endif()

	if(NOT DEFINED GEH_EXPORT_MACRO_NAME)
		set(GEH_EXPORT_MACRO_NAME "${GEH_PREFIX_NAME}${GEH_BASE_NAME}_EXPORT")
	endif()
	if(NOT DEFINED GEH_EXPORT_MACRO_CNAME)
		set(GEH_EXPORT_MACRO_CNAME "${GEH_PREFIX_NAME}${GEH_BASE_NAME}_CEXPORT")
	endif()
	if(NOT DEFINED GEH_NO_EXPORT_MACRO_NAME)
		set(GEH_NO_EXPORT_MACRO_NAME "${GEH_PREFIX_NAME}${GEH_BASE_NAME}_NO_EXPORT")
	endif()

	if(NOT DEFINED GEH_DEPRECATED_MACRO_NAME)
		set(GEH_DEPRECATED_MACRO_NAME "${GEH_PREFIX_NAME}${GEH_BASE_NAME}_DEPRECATED")
	endif()
	if(NOT DEFINED GEH_NO_DEPRECATED_MACRO_NAME)
		set(GEH_NO_DEPRECATED_MACRO_NAME "${GEH_PREFIX_NAME}${GEH_BASE_NAME}_NO_DEPRECATED")
	endif()

	if(DEFINED GEH_CUSTOM_CONTENT_FROM_VARIABLE)
		if(DEFINED ${GEH_CUSTOM_CONTENT_FROM_VARIABLE})
			set(GEH_CUSTOM_CONTENT "${${GEH_CUSTOM_CONTENT_FROM_VARIABLE}}")
		endif()
	endif()

	if(NOT DEFINED _CURRENT_YEAR)
		string(TIMESTAMP _CURRENT_YEAR "%Y")
		string(TIMESTAMP _CURRENT_DATE "%Y%m%d")
	endif()

	configure_file("${NXGenerateExportHeader_DIRECTORY}/NXGenerateExportHeader.configure" "${GEH_EXPORT_FILE_NAME}")

	nx_scope_propagate()
endfunction()
