# -------------------------------
# SPDX-License-Identifier: ISC
#
# Copyright © 2021 Daniel Wolf <<nephatrine@gmail.com>>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
# -------------------------------

include(NXIncludeGuard)
include(NXScopePropagate)

nx_include_guard()

# ===================================================================

set(NXGetRuntimeDependencies_DIRECTORY "${CMAKE_CURRENT_LIST_DIR}")

function(nx_add_runtime_executable sTargetName)
	nx_scope_initialize()

	if(TARGET ${sTargetName})
		list(APPEND _NX_PROPAGATE_GLOBAL "_NX_GATHER_EXECUTABLES")
		list(APPEND _NX_PROPAGATE_SORT "_NX_GATHER_EXECUTABLES")
		list(APPEND _NX_GATHER_EXECUTABLES "${sTargetName}")
	endif()

	nx_scope_propagate()
endfunction()

function(nx_add_runtime_library sTargetName)
	nx_scope_initialize()

	if(TARGET ${sTargetName})
		list(APPEND _NX_PROPAGATE_GLOBAL "_NX_GATHER_LIBRARIES")
		list(APPEND _NX_PROPAGATE_SORT "_NX_GATHER_LIBRARIES")
		list(APPEND _NX_GATHER_LIBRARIES "${sTargetName}")
	endif()

	nx_scope_propagate()
endfunction()

function(nx_get_runtime_dependencies)
	nx_scope_initialize()

	if(WIN32 AND CMAKE_VERSION VERSION_GREATER_EQUAL "3.16")
		unset(GRD_TARGET_LIST)
		unset(GRD_SYSTEM_LIST)

		if(DEFINED _NX_GATHER_EXECUTABLES)
			set(GRD_TARGET_LIST "${GRD_TARGET_LIST}EXECUTABLES ")
			foreach(sTargetName ${_NX_GATHER_EXECUTABLES})
				set(GRD_TARGET_LIST "${GRD_TARGET_LIST}$<TARGET_FILE:${sTargetName}> ")
			endforeach()
		endif()
		if(DEFINED _NX_GATHER_LIBRARIES)
			set(GRD_TARGET_LIST "${GRD_TARGET_LIST}LIBRARIES ")
			foreach(sTargetName ${_NX_GATHER_LIBRARIES})
				set(GRD_TARGET_LIST "${GRD_TARGET_LIST}$<TARGET_FILE:${sTargetName}> ")
			endforeach()
		endif()

		foreach(sPathName ${CMAKE_SYSTEM_LIBRARY_PATH} ${CMAKE_MINGW_SYSTEM_LIBRARY_PATH})
			set(GRD_SYSTEM_LIST "${GRD_SYSTEM_LIST}\"${sPathName}\" ")
		endforeach()

		if(NOT DEFINED _CURRENT_YEAR)
			string(TIMESTAMP _CURRENT_YEAR "%Y")
			string(TIMESTAMP _CURRENT_DATE "%Y%m%d")
		endif()

		configure_file("${NXGetRuntimeDependencies_DIRECTORY}/NXGetRuntimeDependencies.configure" "_NXGetRuntimeDependencies.cmake" @ONLY)
		list(APPEND CMAKE_MODULE_PATH "${CMAKE_CURRENT_BINARY_DIR}")
		include(_NXGetRuntimeDependencies)
	endif()

	nx_scope_propagate()
endfunction()
