# -------------------------------
# SPDX-License-Identifier: ISC
#
# Copyright © 2021 Daniel Wolf <<nephatrine@gmail.com>>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
# -------------------------------

macro(nx_scope_initialize)
	if(NOT DEFINED _NX_SCOPE)
		set(_NX_SCOPE ON)
	else()
		set(_NX_PROPAGATE ON)
	endif()
endmacro()

macro(nx_scope_propagate)
	if(DEFINED _NX_PROPAGATE_SORT AND _NX_PROPAGATE_SORT)
		list(REMOVE_DUPLICATES _NX_PROPAGATE_SORT)
	endif()

	foreach(vPropagate "_NX_PROPAGATE_RESET" "_NX_PROPAGATE_GLOBAL" "_NX_PROPAGATE_SET" "_NX_PROPAGATE_LOCAL" ${_NX_PROPAGATE_SORT})
		if(DEFINED ${vPropagate} AND ${vPropagate})
			list(SORT ${vPropagate})
			list(REMOVE_DUPLICATES ${vPropagate})
		endif()
	endforeach()

	foreach(vPropagate "_NX_PROPAGATE_RESET" "_NX_PROPAGATE_GLOBAL" "_NX_PROPAGATE_SORT" ${_NX_PROPAGATE_GLOBAL} ${_NX_PROPAGATE_SET}
						${_NX_PROPAGATE_LOCAL})
		if(DEFINED ${vPropagate})
			set(${vPropagate}
				"${${vPropagate}}"
				PARENT_SCOPE)
		else()
			unset(${vPropagate} PARENT_SCOPE)
		endif()
	endforeach()

	if(DEFINED _NX_PROPAGATE)
		set(_NX_PROPAGATE_SET
			"${_NX_PROPAGATE_SET}"
			PARENT_SCOPE)
	endif()
endmacro()

macro(nx_scope_project)
	if(NOT CMAKE_CURRENT_SOURCE_DIR STREQUAL "${CMAKE_SOURCE_DIR}")
		foreach(vPropagate "_NX_PROPAGATE_RESET" "_NX_PROPAGATE_GLOBAL")
			if(DEFINED ${vPropagate} AND ${vPropagate})
				list(SORT ${vPropagate})
				list(REMOVE_DUPLICATES ${vPropagate})
			endif()
		endforeach()

		foreach(vPropagate "_NX_PROPAGATE_RESET" "_NX_PROPAGATE_GLOBAL" ${_NX_PROPAGATE_GLOBAL})
			if(DEFINED ${vPropagate})
				set(${vPropagate}
					"${${vPropagate}}"
					PARENT_SCOPE)
			else()
				unset(${vPropagate} PARENT_SCOPE)
			endif()
		endforeach()
	endif()
endmacro()

function(nx_set_global vName sValue)
	nx_scope_initialize()
	list(APPEND _NX_PROPAGATE_GLOBAL "${vName}")
	set(${vName} "${sValue}")
	message(STATUS "SET: ${vName} to '${sValue}'")
	nx_scope_propagate()
endfunction()

function(nx_append_global vName sValue)
	nx_scope_initialize()
	list(APPEND _NX_PROPAGATE_GLOBAL "${vName}")
	list(APPEND _NX_PROPAGATE_SORT "${vName}")
	list(APPEND ${vName} "${sValue}")
	nx_scope_propagate()
endfunction()

function(nx_set_project vName sValue)
	nx_scope_initialize()
	list(APPEND _NX_PROPAGATE_SET "${vName}")
	set(${vName} "${sValue}")
	nx_scope_propagate()
endfunction()

function(nx_set_local vName sValue)
	nx_scope_initialize()
	list(APPEND _NX_PROPAGATE_LOCAL "${vName}")
	set(${vName} "${sValue}")
	nx_scope_propagate()
endfunction()
